package externaljava;


import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

public class BirtReportHelper {
    public static final String REPORT_PARAM = "__report";
    public static final String FORMAT_PARAM = "__format";
    public static final String FORMAT_PDF = "pdf";
    public static final String CREATE_PARAM = "createPDF";
    public static final String NULL = "null";
    public static final int EOF = -1;
    private static final int BUFFER_SIZE = 1024 * 4;
    private static final Logger _logger = Logger.getLogger(BirtReportHelper.class);

    /**
     * Main method, that downloads report file from BIRT server
     *
     * @param birtHost       BIRT host path. Must starts with "http(s)://" (example: "http://meatcutsheet.com:8080/birt/").
     * @param birtReportName full path to the .rptdesign file (example: "/opt/birt/CutSheetReport.rptdesign")
     * @param reportParams   Map<String,String> with report's mandatory params
     * @param storeCopy      flag, that indicates that we need to store report copy in default dir.
     * @return java.io.File that points to the generated report, or null - if some exception occurred
     */
    public static File getBirtReportFile(String birtHost, String birtReportName, Map<String, String> reportParams, boolean storeCopy) {
        File file = null;
        InputStream inputStream = null;
        FileOutputStream outputStream = null;
        try {
            _logger.debug("[BRH] Report creation started! Extra storage requested: "+storeCopy);
            file = File.createTempFile(REPORT_PARAM, '.' + FORMAT_PDF);
            inputStream = new BufferedInputStream(new URL(createURL(birtHost, birtReportName, reportParams, storeCopy)).openStream());
            outputStream = new FileOutputStream(file);
            copyStreams(inputStream, outputStream);
            _logger.debug("[BRH] Report creation finished.\n Real Path:"+file.getAbsolutePath());
        } catch (IOException e) {
            _logger.error("[BRH] Failed to get BIRT report. Cause:" + e.getMessage(), e);
        } finally {
            closeStreams(inputStream, outputStream);
        }

        return file;
    }

    public static File getBirtReportFile(String birtHost, String birtReportName, String[] reportParams, boolean storeCopy) {
        return getBirtReportFile(birtHost, birtReportName, createMapFromArray(reportParams), storeCopy);
    }

    /**
     * @param birtHost       BIRT host path. Must starts with "http(s)://" (example: "http://meatcutsheet.com:8080/birt/").
     * @param birtReportName full path to the .rptdesign file (example: "/opt/birt/CutSheetReport.rptdesign")
     * @param reportParams   Map<String,String> with report's mandatory params
     * @param storeCopy      flag, that indicates that we need to store report copy in default dir.
     * @return string that contains full path to created report file, or empty string
     */
    public static String getBirtReportFilePath(String birtHost, String birtReportName, Map<String, String> reportParams, boolean storeCopy) {
        File report = getBirtReportFile(birtHost, birtReportName, reportParams, storeCopy);
        return report == null ? "" : report.getAbsolutePath();
    }

    public static String getBirtReportFilePath(String birtHost, String birtReportName, String[] reportParams, boolean storeCopy) {
        return getBirtReportFilePath(birtHost, birtReportName, createMapFromArray(reportParams), storeCopy);
    }

    private static String createURL(String host, String reportName, Map<String, String> params, boolean storeCopy) {
        StringBuilder url = new StringBuilder();
        url.append(host).append("/run?")
                .append(REPORT_PARAM).append("=").append(reportName).append("&")
                .append(FORMAT_PARAM).append("=").append(FORMAT_PDF).append("&");
        if (isNotEmpty(params)) {
            for (Map.Entry<String, String> entry : params.entrySet()) {
                url.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
            }
        }
        url.append(CREATE_PARAM).append("=").append(storeCopy);
        String result = url.toString();
        _logger.warn("[BRH] REPORT URL: " + result);
        return result;
    }

    /**
     * Method that copies input stream to output, without closing them
     *
     * @param input  source stream
     * @param output destination stream
     * @throws IOException
     */
    public static void copyStreams(InputStream input, OutputStream output) throws IOException {
        if (input != null && output != null) {
            byte[] buffer = new byte[BUFFER_SIZE];
            int length = 0;
            while (EOF != (length = input.read(buffer))) {
                output.write(buffer, 0, length);
            }
            output.flush();
        }
    }


    private static void closeStreams(InputStream input, OutputStream output) {
        try {
            if (input != null) {
                input.close();
            }
            if (output != null) {
                output.close();
            }
        } catch (IOException e) {
            _logger.warn("[BRH] Failed to close streams. Cause:" + e.getMessage(), e);
        }
    }


    //    Convertation utils
    public static Map<String, String> createMapFromArray(String[] strings) {
        if (strings == null || strings.length == 0) {
            return Collections.emptyMap();
        }
        Map<String, String> result = new HashMap<String, String>();
        int size = strings.length;
        for (int i = 0; i < size; i += 2) {
            String key = strings[i];
            String value = size > i + 1 ? strings[i + 1] : null;
            result.put(key, value);
        }
        return result;
    }

    public static Map<String, String> createMapFromVarArgs(String... strings) {
        return createMapFromArray(strings);
    }

    public static Map<String, String> createMapFromList(List<String> stringList) {
        if (stringList == null || stringList.isEmpty()) {
            return Collections.emptyMap();
        }
        Map<String, String> result = new HashMap<String, String>();
        int size = stringList.size();
        for (int i = 0; i < size; i += 2) {
            String key = stringList.get(i);
            String value = size > i + 1 ? stringList.get(i + 1) : null;
            result.put(key, value);
        }
        return result;
    }

    public static <T> String join(List<T> list, String separator) {
        StringBuilder builder = new StringBuilder();
        separator = isEmpty(separator) ? "," : separator;
        if (isNotEmpty(list)) {
            for (T t : list) {
                builder.append(t == null ? NULL : t.toString()).append(separator);
            }
        }
        return builder.toString();
    }

    public static <K, V> String join(Map<K, V> map, String separator) {
        StringBuilder builder = new StringBuilder();
        separator = isEmpty(separator) ? "," : separator;
        if (isNotEmpty(map)) {
            for (Map.Entry<K, V> e : map.entrySet()) {
                if (e != null) {
                    builder.append("key:").append(e.getKey() == null ? NULL : e.getKey().toString()).append(",value:").append(e.getValue() == null ? NULL : e.getValue().toString()).append(separator);
                }
            }
        }
        return builder.toString();
    }


    //    Check utils
    public static boolean isEmpty(String s) {
        return s == null || "".equals(s.trim());
    }

    public static boolean isNotEmpty(String s) {
        return !isEmpty(s);
    }

    public static <T> boolean isEmpty(Collection<T> collection) {
        return collection == null || collection.isEmpty();
    }

    public static <T> boolean isNotEmpty(Collection<T> collection) {
        return !isEmpty(collection);
    }

    public static <K, V> boolean isEmpty(Map<K, V> map) {
        return map == null || map.isEmpty();
    }

    public static <K, V> boolean isNotEmpty(Map<K, V> map) {
        return !isEmpty(map);
    }


    public static List<String> createListFromArray(String[] strings) {
        if (strings == null || strings.length == 0) {
            return Collections.emptyList();
        }
        return Arrays.asList(strings);
    }

}
